@@ -11,12 +11,33 @@ use DateTime::SpanSet;
 
 use Carp;
 use Params::Validate qw( validate SCALAR BOOLEAN OBJECT CODEREF ARRAYREF );
-use Set::Infinite '0.44';
-$Set::Infinite::PRETTY_PRINT = 1;   # enable Set::Infinite debug
+
+# use Set::Infinite '0.44';
+# $Set::Infinite::PRETTY_PRINT = 1;   # enable Set::Infinite debug
 
 use constant INFINITY     =>       100 ** 100 ** 100 ;
 use constant NEG_INFINITY => -1 * (100 ** 100 ** 100);
 
+sub set_time_zone {
+    my ( $self, $tz ) = @_;
+
+    my $result = $self->{set}->iterate( 
+        sub {
+            $_[0]{list}[0]{a}->set_time_zone( $tz ) if ref $_[0]{list}[0]{a};
+            $_[0]{list}[0]{b}->set_time_zone( $tz ) if ref $_[0]{list}[0]{b};
+        }
+    );
+
+    ### this code enables 'subroutine method' behaviour
+    $self->{set} = $result;
+    return $self;
+
+    ### this code enables 'function method' behaviour
+    # my $set = $self->clone;
+    # $set->{set} = $result;
+    # return $set;
+}
+
 sub from_spans {
     my $class = shift;
     my %args = validate( @_,
@@ -27,7 +48,7 @@ sub from_spans {
                          }
                        );
     my $self = {};
-    my $set = Set::Infinite->new();
+    my $set = Set::Infinite::_recurrence->new();
     $set = $set->union( $_->{set} ) for @{ $args{spans} };
     $self->{set} = $set;
     bless $self, $class;
@@ -70,7 +91,7 @@ sub from_sets {
 sub empty_set {
     my $class = shift;
 
-    return bless { set => Set::Infinite->new }, $class;
+    return bless { set => Set::Infinite::_recurrence->new }, $class;
 }
 
 sub clone { 
@@ -184,9 +205,8 @@ sub duration { my $dur = $_[0]->{set}->size; defined $dur ? $dur : INFINITY }
 *size = \&duration;
 
 # unsupported Set::Infinite methods
-
-sub offset { die "offset() not supported"; }
-sub quantize { die "quantize() not supported"; }
+# sub offset { die "offset() not supported"; }
+# sub quantize { die "quantize() not supported"; }
 
 1;
 
@@ -272,6 +292,22 @@ a span like C<[$dt, inf)>.
 
 Creates a new empty set.
 
+=item * clone
+
+This object method returns a replica of the given object.
+
+=item * set_time_zone( $tz )
+
+This method accepts either a time zone object or a string that can be
+passed as the "name" parameter to C<< DateTime::TimeZone->new() >>.
+If the new time zone's offset is different from the old time zone,
+then the I<local> time is adjusted accordingly.
+
+If the old time zone was a floating time zone, then no adjustments to
+the local time are made, except to account for leap seconds.  If the
+new time zone is floating, then the I<UTC> time is adjusted in order
+to leave the local time untouched.
+
 =item * min / max
 
 First or last dates in the set.  These methods may return C<undef> if
